
-- return the window size
getWindowSize = getWindowSize or function()
	return 786, 257
end

-- create and lay out all the controls
createUI = createUI or function()

	createUICommon()

	-- constants
	local kSpace = 19
	local kKnobSpace = kKnobSize + kSpace
	local kKnobSpaceV = 70
	local kPropertyTextFieldH = 20
	local kLabelSpaceV = 30
	local kTextW = 40

	local function knobGroup( LR, title, id )
		g = group( title )
		local x = 0
		local y = 0
		
		g:add( paramButton( "Enable " .. LR,					"Enable",				x, y, 50, 30 ) )
		x = x + 50 + pad
		labelledDropDown( g, "Mode " .. LR,						"Mode",					x, y )
		x = x + kDropdownW + pad
		smallKnobAndTextField( g, "Steps " .. LR,				"Steps",				x, y )
		x = x + kKnobSpace
		smallKnobAndTextField( g, "Steps Fine " .. LR,			"Fine",					x, y, kTextW, "%.3f" )
		x = x + kKnobSpace
		smallKnobAndTextField( g, "Offset " .. LR,				"Offset",				x, y )
		x = x + kKnobSpace
		smallKnobAndTextField( g, "Transpose " .. LR,			"Transpose",			x, y )
		x = x + kKnobSpace
		g:add( paramButton( "Enable Scale " .. LR,				"Scale",				x, y, 50, 30 ) )
		x = x + 50 + pad
		local kScaleDropdownW = 90
		labelledDropDown( g, "Key " .. LR,						"Key",					x, y + 30, kScaleDropdownW )
		labelledDropDown( g, "Scale " .. LR,					"Scale",				x, y, kScaleDropdownW )
		x = x + kScaleDropdownW + kSpace + 10
		smallKnobAndTextField( g, "Key Offset " .. LR,			"Key Offset",			x, y, kTextW )
		x = x + kKnobSpace + 10
		smallKnobAndTextField( g, "Smoothing " .. LR,			"Smooth",				x, y )
		x = x + kKnobSpace
		
		g:add( quantizerInfoDisplay( x, y, 49, 30, id ) )

		x = 0
		y = kKnobSpaceV
	
		return g
	end

	-- make groups and knobs
	--

	theKnobRenderer = smallKnobRenderer

	miscGroup = group( "OSC" )
	smallKnobAndTextField( miscGroup, "OSC Port Offset",	"Port",		0*kKnobSpace, 0 )
	miscGroup:setVisible( false )

	leftGroup = knobGroup( "L", "Left Channel", 0 )
	rightGroup = knobGroup( "R", "Right Channel", 1 )

	-- positions!

	local xbase = windowEdgePad
	local ybase = windowEdgePad
	local x = xbase
	local y = ybase
	
	rightGroup:setPosition( x, y, true )
	x = x + rightGroup:width() + pad
	
	miscGroup:setPosition( x, y, true )
	x = x + miscGroup:width() + pad
	
	x = xbase
	y = y + rightGroup:height() + pad
	
	local filenameY = y
	y = y + kPropertyTextFieldH + pad
	
	leftGroup:setPosition( x, y, true )
	x = x + leftGroup:width() + pad
	
	local prefsX = x
	local prefsY = y

	-- add directly positioned objects
	--

	-- add the prefs button
	
	local kPrefsW = 60
	y = prefsY + 40

	add( button( "QuantizerLoad", "Load", prefsX, y, kPrefsW, 30 ) )
	y = y - 40

	add( propertyTriggerButton( 64090, "ES-4", "Create a calibration suitable for the ES-4", prefsX, y, kPrefsW, 30 ) )
	y = y - 40

	add( propertyTriggerButton( 64091, "ESX-4CV", "Create a calibration suitable for the ESX-4CV", prefsX, y, kPrefsW, 30 ) )

	local kLabelW = 140
	local lab = label( "Calibration loaded from:", xbase, filenameY+0.5*(kPropertyTextFieldH-kTextFieldH), "center", kLabelW, kTextFieldH )
	setLabelColour( lab )
	add( lab )
	field = propertyTextField( 64121, "Loaded calibration file", xbase+kLabelW+pad, filenameY, rightGroup:width()-kLabelW-pad, kPropertyTextFieldH )
	field:setEditable( false )
	add( field )

end		-- of createUI()
